### -*-sh-*-
###
### Functions for backup clients.
###
### (c) 2011 Mark Wooding
###

###----- Licensing notice ---------------------------------------------------
###
### This file is part of the distorted.org.uk backup suite.
###
### distorted-backup is free software; you can redistribute it and/or modify
### it under the terms of the GNU General Public License as published by
### the Free Software Foundation; either version 2 of the License, or
### (at your option) any later version.
###
### distorted-backup is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with distorted-backup; if not, write to the Free Software Foundation,
### Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

set -e

###--------------------------------------------------------------------------
### Utilities.

QUIS=${0##*/}
fail () { echo >&2 "$QUIS: $*"; exit 1; }

preflight () {
  for i in TMPDIR HOST RMT LEVEL LASTDATE ASSET TARGET; do
    eval "p=\${BKP_$i+t}"
    case "$p" in t) ;; *) fail "environment not correctly configured" ;; esac
  done
}

run () {
  echo "$*"
  "$@"
}

bkpadmin () {
  ## bkpadmin COMMAND ARGUMENT ...
  ssh $BKP_HOST userv root bkpadmin "$@"
}

datefmt () {
  fmt=$1 date=$2
  ## Convert the DATE (in ISO8601-ish format, with an optional numeric
  ## timezone offset) according to the strftime(3) format FMT.

  ## We have Perl available, so you'd think this would be fairly easy.  Alas,
  ## not.  The obvious thing to do would be use the Date::Format module, but
  ## that's unusably broken.  Consider:
  ##
  ##   $t0 = 1319934600; $t1 = $t0 + 3600;	# obviously different
  ##   for my $i ($t0, $t1)			# print identically
  ##     { print time2str "%a %b %e %H:%M:%S %Y %z\n", $i; }
  ##
  ## The Date::Parse module seems to work correctly, but isn't installed by
  ## default on some of our target platforms.  So we end up doing a lot of
  ## the work by hand.

  perl -e '
    use POSIX;

    my ($fmt, $date) = @ARGV;

    ## Parse the input date.
    my ($yr, $mo, $dy, $hr, $mi, $s, $tz) = $date =~
      /^\s*(\d+)-(\d+)-(\d+)\s+(\d+):(\d+):(\d+)(?:\s+)?([-+]\d+)?\s*$/
      or die "bad input date `$date'\''";

    ## Convert the input date into a time_t.  This is annoyingly fiddly.  If
    ## an explicit timezone offset is supplied, do the conversion as UTC, and
    ## then apply the timezone correction.  This means that we must hack
    ## about with the awful BCD timezone offset.
    my $t;
    if (!defined $tz) {
      $t = mktime $s, $mi, $hr, $dy, $mo - 1, $yr - 1900, undef, undef, -1;
    } else {
      use integer;
      my ($tzsign, $tzabs) = $tz < 0 ? (-1, -$tz) : (+1, $tz);
      my $tzoff = $tzsign*(60*($tzabs/100) + ($tzabs%100));
      local $ENV{TZ} = "UTC0"; tzset;
      $t = mktime $s, $mi, $hr, $dy, $mo - 1, $yr - 1900, undef, undef, 0;
      $t -= 60*$tzoff;
    }

    ## Now format this as requested.
    tzset; my @tm = localtime $t;
    print strftime $fmt, @tm;
    print "\n";
  ' "$fmt" "$date"
}

###----- That's all, folks --------------------------------------------------
